  Document.prototype.__defineGetter__("xml", function ()
  {
   return (new XMLSerializer()).serializeToString(this);
  });

  Document.prototype.loadFromString = function (s) {

   // parse the string to a new doc
   var doc2 = (new DOMParser()).parseFromString(s, "text/xml");

   // remove all initial children
   while (this.hasChildNodes())
      this.removeChild(this.lastChild);

   // insert and import nodes
   for (var i = 0; i < doc2.childNodes.length; i++) {
      this.appendChild(this.importNode(doc2.childNodes[i], true));
   }
  }

  function getSiblingNode(searchnode, nodename)
  {
    parentnode = searchnode;
    var result = null;
    var nodeidx = 0;
    var nodecount = 0;

    // determine if the nodename contains an index
    if (nodename.indexOf('[')!=-1 && nodename.indexOf(']')!=-1)
    {
      idxpart = nodename.substring(nodename.indexOf('[')+1, nodename.indexOf(']'));
      nodeidx = idxpart;
      nodename = nodename.substring(0, nodename.indexOf('['));
    }

    if (nodename == '')
    {
      // empty search
      return parentnode;
    }
    else if (nodename.substr(0,1)=='@')
    {
      // search for attributes
      var attrstr = nodename.substr(1,nodename.length-1);
      if (!parentnode.getAttribute)
      {
        return '';
      }
      else
        return parentnode.getAttribute(attrstr);
    }
    else if (parentnode.hasChildNodes())
    {
      // search for child nodes
      currentnode = parentnode.firstChild;
      
      while (currentnode)
      {
        if (currentnode.nodeName==nodename)
        {
          if (nodeidx == nodecount)
            return(currentnode);
          nodecount++;
        }
        currentnode = currentnode.nextSibling;
      }
      
    }
    return result;
  }

  Document.prototype.getSubDocument = function (pathstr)
  {
    var nodes = pathstr.split('/');
    var resultstr = '';

    if (nodes.length > 0)
    {
      basenode = this;
      for (a = 1; a < nodes.length; a++)
        if (basenode)
        {
          basenode = getSiblingNode(basenode,nodes[a]);
        }
      if (basenode)
      {
        // found!
        resultstr = basenode;
        if (basenode.nodeType)
        {
          resultstr = document.implementation.createDocument("", "", null);
          resultstr.appendChild(basenode);
        }
      }
    }
    return resultstr;
  }                                     

  Document.prototype.setNodeElement = function (pathstr, newElement, createifnotexists)
  {
    var nodes = pathstr.split('/');
    var prevnode = this;
                                                         
    if (newElement != null)
      newElement=newElement.firstChild;
  
    if (nodes.length > 0)
    {
      basenode = this;
      for (a = 1; a < nodes.length; a++)
        if (basenode && nodes[a].substr(0,1)!='@')
        {
          prevnode = basenode;
          try
          {
             basenode = getSiblingNode(basenode,nodes[a]);
          }
          catch(ex)
          {
              basenode=null;
          }
          if (createifnotexists == true && basenode == null)
          {
            if (nodes[a].substr(0,1)=='@')
            {
              // is attribute
              var attrname = nodes[a].substr(1, nodes[a].length-1);
              var attrnode = this.createAttribute(attrname);
              attrnode.nodeValue = newElement;
              prevnode.setAttributeNode(attrnode);
            }
            else
            {
              // node
              // determine if the nodename contains an index
              var nodename = nodes[a];
              if (nodename.indexOf('[')!=-1 && nodename.indexOf(']')!=-1)
              {
                nodename = nodename.substring(0, nodename.indexOf('['));
              }
              basenode = this.createElement(nodename);
              prevnode.appendChild(basenode);
            }
          }
        }
      if (basenode)
      {
        // found!
        if (basenode.nodeType && nodes[nodes.length-1].substr(0,1)!='@')
        {
          if (basenode.firstChild)
          {
            if (newElement == null)
            {
              if (createifnotexists == true)
              {
                nextnode=basenode.firstChild;
                while(nextnode!=null)
                {                
                  basenode.removeChild(nextnode);
                  nextnode=basenode.firstChild;
                }
               }
               else
               {
                  prevnode=basenode.parentNode;
                  prevnode.removeChild(basenode);
              }
            }
            else
            {
                basenode.appendChild(newElement);
            }
          }
          else
          {
             if (newElement != null)
               basenode.appendChild(newElement);
          }
          return true;
        }
        else
        {
          var attrstr = nodes[nodes.length-1].substr(1,nodes[nodes.length-1].length-1);
          basenode.setAttribute(attrstr, newElement);
          return true;
        }
      }
    }
    return false;
  }

  Document.prototype.placeSubDocument = function (pathstr,doc,overwrite)
  {
    var nodes = pathstr.split('/');
    var resultstr = this.xml;
    for (a = 1; a < nodes.length-1; a++)
    {
      resultstr=resultstr.substring(resultstr.indexOf(nodes[a])+nodes[a].length);
      resultstr=resultstr.substring(resultstr.indexOf('>')+1); 
    }
    var bString = this.xml.substring(0,this.xml.indexOf(resultstr));
    if ((resultstr.indexOf(nodes[nodes.length-1]) > -1) && overwrite)
    {
      fStr=resultstr.substr(0,resultstr.indexOf('<'+nodes[nodes.length-1]+'>'));
      sStr=resultstr.substr(resultstr.indexOf('</'+nodes[nodes.length-1]+'>')+nodes[nodes.length-1].length+3);
      resultstr=fStr+sStr;
    }
    var pStr=bString + doc.xml + resultstr;
    this.loadFromString(pStr);
    return true;
  }
  
  
  function element2Doc(element)
  {
    result = document.implementation.createDocument("", "", null);
    try
    {
      if (element)
        result.appendChild(element);
    }
    catch (Exception)
    {
    
    }
    return(result);  
  }

  Document.prototype.getNodeValue = function (pathstr)
  {
    var nodes = pathstr.split('/');
    var resultstr = '';

    if (nodes.length > 0)
    {
      basenode = this;
      for (a = 1; a < nodes.length; a++)
        if (basenode)
        {
          basenode = getSiblingNode(basenode,nodes[a]);
        }
      if (basenode)
      {
        // found!             
        resultstr = basenode;
        if (basenode.nodeType)
        { 
          if (basenode.firstChild)
          {  
            if (basenode.nodeName == '#text')
            {
              resultstr = datanode.data;
            }
            else
            {
              found=false;
              resultstr = '';              
              datanode=basenode.firstChild;
              while (datanode && !found)
              {
                if(datanode.nodeName != '#text')
                  datanode=datanode.nextSibling;
                else 
                {
                  found = true;
                  resultstr = datanode.data;
                }
              }
            }
          }
          else
          {
            resultstr = '';
          }
        }
      }
    }
    return resultstr;
  }

  Document.prototype.setNodeValue = function (pathstr, vstr, createifnotexists)
  {

    //pathstr=pathstr.toLowerCase();
    var nodes = pathstr.split('/');
    var prevnode;

    if (nodes.length > 0)
    {
      basenode = this;
      //alert(basenode.nodeName+' -> '+pathstr);
      for (a = 1; a < nodes.length; a++)
        if (basenode && nodes[a].substr(0,1)!='@')
        {
          prevnode = basenode;
          try
          {
             basenode = getSiblingNode(basenode,nodes[a]);
          }
          catch(ex)
          {
              basenode=null;
          }
          if (createifnotexists == true && basenode == null)
          {
            if (nodes[a].substr(0,1)=='@')
            {
              // is attribute
              var attrname = nodes[a].substr(1, nodes[a].length-1);
              var attrnode = this.createAttribute(attrname);
              attrnode.nodeValue = vstr;
              prevnode.setAttributeNode(attrnode);
            }
            else
            {
              // node
              // determine if the nodename contains an index
              var nodename = nodes[a];
              if (nodename.indexOf('[')!=-1 && nodename.indexOf(']')!=-1)
              {
                nodename = nodename.substring(0, nodename.indexOf('['));
              }
              basenode = this.createElement(nodename);
              //alert(nodename);
              prevnode.appendChild(basenode);
            }
          }
        }
      if (basenode)
      {
        // found!
        if (basenode.nodeType && nodes[nodes.length-1].substr(0,1)!='@')
        {
          if (basenode.firstChild)
          {
            if (vstr == null)
            {
                nodename=basenode.nodeName;
                prevnode.removeChild(basenode);
                addnode = this.createElement(nodename);
                prevnode.appendChild(addnode);
            }
            else
            {
                basenode.firstChild.data = vstr;
            }
          }
          else
          {
            tn = this.createTextNode(vstr);
            basenode.appendChild(tn);
          }
          return true;
        }
        else
        {
          var attrstr = nodes[nodes.length-1].substr(1,nodes[nodes.length-1].length-1);
          basenode.setAttribute(attrstr, vstr);
          return true;
        }
      }
    }
    return false;
  }


  function getDOMObj(binding)
  {
      var nodes = binding.split('/');
      var result=null;
      var sources=new Array();
      
      for (var i=0; i<datasources.length; i++)
      {
        xmlObj=datasources[i];
        //alert(i);
        //alert(xmlObj.xml);
        if (xmlObj.firstChild.nodeName == nodes[1])
        {
            result=xmlObj;
        }
        else
        {
            sources.push(datasources[i]);
        }
      }
      datasources=sources;
      return result;
  }

  function selectDOMObj(binding)
  {
      var nodes = binding.split('/');
      for (var i=0; i<datasources.length; i++)
      {
        xmlObj=datasources[i];
        if (xmlObj.firstChild.nodeName == nodes[1])
        {
          //alert(xmlObj.firstChild.nodeName);
          return(xmlObj);          
        }
      }
      return null;
  }

  function markField(colorcode)
  {
      tObj = document.popupNode;
      mainElementId = tObj.getAttribute('id');
      mainElementId = mainElementId.substr(0,mainElementId.length-5);
      mainElementObj = document.getElementById(mainElementId);
        
      //Change font color 
      //
      objStyle = tObj.getAttribute('style');
      objStyle += ' color:'+colorcode;
      tObj.setAttribute("style", objStyle);
      objStyle = mainElementObj.getAttribute('style');
      objStyle += ' color:'+colorcode;
      mainElementObj.setAttribute("style", objStyle);
        
      //Store color to DOM
      //
      if (mainElementObj.getAttribute("datasource") != "")
      {
          objToWrite = selectDOMObj('/' + mainDomName);
          var binding = '/' + mainDomName + mainElementObj.getAttribute("binding");
      }
      else
      {
          objToWrite = selectDOMObj(mainElementObj.getAttribute("binding"));
          var binding = mainElementObj.getAttribute("binding");
      }
      if (objToWrite != null) 
      {
          binding = binding + '/@color';
          objToWrite.setNodeValue(binding, colorcode, true);
          //alert('wrote color to '+binding);
      }
  }
