<?
  /* File:    lib/contentnode.class.php: provides the ContentNode class that serves
   *          as the base for any operations regarding document nodes.
   * Type:    CMS class library
   * Author:  udo.schroeter@gmail.com
   * License: commercially licensed as part of the CMS package
   * Todo:    - support for multiple sites
   * Changes: /
   */

  /**
  * NodeStorage is the simple default storage class for non-versioned live content
  */
  class NodeStorage
  {
    function NodeStorage()
    {

    }

    function save($nodeDS)
    {
      global $config;
      if (isset($nodeDS['p_key']) && $config['cms']['versioning']==true)
      {
        // create an undo dataset
        $undoDSNode = DB_GetDataset('pages', $nodeDS['p_key']);
        $undoDS = array(
          'c_ref_node' => $nodeDS['p_key'],
          'c_ref_version' => -1,
          'c_status' => '^',
          'c_node' => serialize($undoDSNode),
          );
        DB_UpdateDataset('versioncontent', $undoDS);
      }
      $key = DB_UpdateDataset('pages', $nodeDS);
      $nodeDS['p_key'] = $key;
      return($nodeDS);
    }

    function load($nodekey)
    {
      return(DB_GetDataset('pages', $nodekey));
    }

    function delete($nodeDS)
    {
      DB_RemoveDataset('pages', $nodeDS['p_key']);
    }
  }


  /**
  * ContentNode encapsulates almost any function that needs to be performed on
  * or with content data.
  */
  class ContentNode
  {
    var $nodeDS;
    var $blocks;
    var $pageTitle;
    var $currentLanguage;
    var $pageInfo;
    var $storageInterface;

    /**
    * creates a new contentnode instance
    *
    * @loadNode	integer	key of the node to retrieve from the database
    */
    function ContentNode($loadNode = null, $nodeDS = null)
    {
      global $config;
      $this->currentLanguage = $this->getCurrentLanguage();
      if ($config['cms']['versioning'] == false)
        $this->storageInterface = new NodeStorage();
      else
      {
        include_once('lib/versioning.php');
        include_once('lib/versionnode.class.php');
        $this->storageInterface = new VersioningStorage();
      }
      if (isset($loadNode))
      {
        $this->load($loadNode);
      }
      else if (isset($nodeDS))
      {
        $this->nodeDS = $nodeDS;
        $this->unserializeDS();
      }
    }

    function getDate($dateFormat = null)
    {
      include_once('lib/conversion.php');
      $dateTime = timeStampToDate($this->nodeDS['p_stamp']);
      if ($dateFormat == null)   
        return(dateTimeToString($dateTime));
      else
        return(date($dateFormat, $dateTime));
    }

    /**
    * initialization for newly created nodes
    *
    * @parent	integer	key of the parent document in the pages table
    */
    function initNewNode($parent = 1)
    {
      global $config;
      include_once('lib/xml2array.php');
      $tmpl = xml2array(implode('', file('etc/newpage_content.xml')));
      $blocks = getNodeValue($tmpl, '/pagecontent/block');
      foreach ($blocks as $block)
      {
        $attr = $block['@'];
        $newBlock = $this->createBlock($attr['section']);
        foreach ($attr as $k => $v)
          $newBlock[$k] = $v;
        foreach ($block as $sectname => $sectcontent)
          if ($sectname != '@')
            $newBlock[$sectname] = str_replace('%13', "\n", $sectcontent[0]['$']);
        $this->setBlockByGuid($newBlock['guid'], $newBlock);
      }
      $this->nodeDS['p_status'] = 'H';
      $this->nodeDS['p_caption'] = i18n('mci.unnamed');
      $this->nodeDS['p_template'] = $config['cms']['defaulttemplate'];
      $this->nodeDS['p_ref_parent'] = $parent;
      $this->nodeDS['p_ref_owner'] = $_SESSION['cmsuser']['us_key'];
    }

    function unserializeDS()
    {
      // unserialize the actual data into memory
      $this->blocks = unserialize($this->nodeDS['p_content']);
      $this->pageInfo = unserialize($this->nodeDS['p_pageinfo']);
      $this->pageTitle = getDefault(
        $this->pageInfo['caption:'.$this->currentLanguage],
        $this->nodeDS['p_caption']);
    }
                                   
    /**
    * loads a given node dataset from the pages table into memory
    *
    * @nodenr	integer	key of the document in the pages table
    */
    function load($nodenr)
    {
      // get the node dataset from storage
      $this->nodeDS = $this->storageInterface->load($nodenr);
      $this->unserializeDS();
    }

    /**
    * saves the current contentnode into the pages table
    *
    * @nodenr	integer	key of the document in the pages table
    */
    function save($nodenr = null)
    {
      if (isset($nodenr)) $nodeDS['p_key'] = $nodenr;
      $this->pageTitle = $this->getPageTitle();
      $this->nodeDS['p_content'] = serialize($this->blocks);
      $this->nodeDS['p_pageinfo'] = serialize($this->pageInfo);
      $this->nodeDS['p_site'] = $config['siteid'];
      $this->nodeDS['p_caption'] = $this->pageTitle;
      $this->nodeDS = $this->storageInterface->save($this->nodeDS);
      return($this->nodeDS['p_key']);
    }

    /**
    * removes the current document from the database
    */
    function delete()
    {
      $this->storageInterface->delete($this->nodeDS);
      //unset($this->nodeDS);
    }

    /**
    * retrieves a list of blocks for the given section identified
    *
    * @section	string section identifier
    */
    function getBlockList($section = 'main')
    {
      $result = array();
      if (is_array($this->blocks[$section]))
        $result = $this->blocks[$section];
      return($result);
    }

    function getFirstBlockId($section = 'main')
    {
      if (is_array($this->blocks[$section]))
        foreach($this->blocks[$section] as $id => $blockItem)
        {
          return($id);
        }
      return(false);
    }

    function getPageTitle()
    {
      global $config;
      if (sizeof($config['site']['language']) > 0)
      foreach ($config['site']['language'] as $lid => $l)
      {
        $lang = $lid;
        break;
      }
      if ($this->pageInfo['caption:'.$lang] != '') return($this->pageInfo['caption:'.$lang]);
      if (!is_array($this->blocks)) $this->blocks = array();
      foreach ($this->blocks as $section)
      {
        foreach ($section as $blockid => $block)
        {
          if ($block['title'] != '') $t = ($block['title']);
          if ($block['title:'.$lang] != '') $t = ($block['title:'.$lang]);
          if ($block['caption'] != '') $t = ($block['caption']);
          if ($block['caption:'.$lang] != '') $t = ($block['caption:'.$lang]);
          if ($t != '')
          {
            return($t);
          }
        }
      }
      return($this->pageTitle);
    }

    /**
    * gets a single block, identified by its uid
    *
    * @guid	string id of the block
    */
    function getBlockByGuid($guid)
    {
      $result = array();
      if (is_array($this->blocks))
        foreach ($this->blocks as $secname => $section)
        {
          if (isset($section[$guid]))
          {
            $block = $section[$guid];
            $block['section'] = $secname;
            return($block);
          }
        }
      return($result);
    }

    /**
    * stores a single block, identified by its uid
    *
    * @guid	string id of the block
    * @saveBlock	array the block (an array of data fields)
    */
    function setBlockByGuid($guid, $saveBlock)
    {
      if ($guid!='')
      {
        $section = getDefault($saveBlock['section'], 'main');
        $this->blocks[$section][$guid] = $saveBlock;
      }
      else
        logToFile('err.node.log', 'Cannot set block without valid ID: '.
          getArrayDump($saveBlock));
    }

    /**
    * counts the blocks in a given section
    *
    * @section string name of the section
    */
    function blockCount($section = 'main')
    {
      return(sizeof($this->blocks[$section]));
    }

    /**
    * add a block to a section
    *
    * @newBlock	array the block
    * @section string section identifier
    */
    function addBlock($newBlock, $section = 'main')
    {
      $newBlock['guid'] = getDefault($newBlock['guid'], md5(microtime()));
      $this->blocks[$section][$newBlock['guid']] = $newBlock;
    }

    /**
    * remove a block, identified by its guid
    *
    * @guid string guid of the block
    */
    function deleteBlockByGuid($guid)
    {
      if (is_array($this->blocks))
        foreach ($this->blocks as $secname => $section)
        {
          if (isset($section[$guid]))
          {
            unset($this->blocks[$secname][$guid]);
            return(true);
          }
        }
      return(false);
    }

    /**
    * create a new block inside a section
    *
    * @section string section identifier
    * @type string block type
    */
    function createBlock($section = 'main', $type = null)
    {
      global $config;
      $type = getDefault($type, $config['cms']['defaultblocktype']);
      $typeDS = DB_GetDataset('blocks', $type);
      $typeName = getDefault($typeDS['b_caption'], $type);
      $result = array(
        'type' => $type,
        'internalname' => $typeName,
        'guid' => md5(microtime()),
        'user' => $_SESSION[$config['cms']['sessioninstance']]['user']['us_key'],
        'style' => $type.'.php',
        );
      $this->addBlock($result, $section);
      return($result);
    }

    /**
    * re-orders the blocks inside a section, given a newly arranged list of uids
    *
    * @listOfGuids array list representing the new order of the blocks with uids
    * @section string name of the section in which to re-arrange
    */
    function orderBlocks($listOfGuids, $section)
    {
      foreach ($listOfGuids as $k=>$v)
        if ($v != '')
        {
          $block = $this->getBlockByGuid($v);
          $blocks[$v] = $block;
        }
      $this->blocks[$section] = $blocks;
    }

    /**
    * get the currently active language
    */
    function getCurrentLanguage()
    {
      global $config;
      $_SESSION['currentlang'] = getDefault($_REQUEST['lang'],
        $_SESSION['currentlang']);
      $result = getDefault($_SESSION['currentlang'],
        $config['site']['defaultlanguage']);
      return($result);
    }

    /**
    * set the currently active language
    *
    * @language string language identifier
    */
    function setCurrentLanguage($language)
    {
      $_SESSION['currentlang'] = $language;
    }

    /**
    * retrieve the current page template's manifest filename
    */
    function getManifestFilename()
    {
      global $config;
      $manifest = getDefault($this->nodeDS['p_template'], 'default.manifest.php');
      return('themes/'.$config['site']['theme'].'/pages/'.$manifest);
    }

  }

  function getNodeCaption(&$nodeDS, $lang = null)
  {
    if ($lang == null) $lang = $_SESSION['currentlang'];
    $pageInfo = unserialize($nodeDS['p_pageinfo']);
    return(getDefault($pageInfo['caption:'.$lang], $nodeDS['p_caption']));
  }

?>
