<?
  /* File:    lib/pagerenderer.class.php: this class takes a ContentNode and
   *          renders a page from it
   * Type:    CMS class library
   * Author:  udo.schroeter@gmail.com
   * License: commercially licensed as part of the CMS package
   * Todo:    /
   * Changes: - support for multiple sites
   */
 
  function stdBreadCrumbItems(&$nodesArray, $separator = ' | ')
  {
    global $currentLanguage;
    $nodes = array();
    $nodesArray[0]['p_caption'] = 'Home';
    foreach ($nodesArray as $item)
      if ($item['p_status'] != 'H')
      {
        $pageInfo = unserialize($item['p_pageinfo']);
        $caption = getDefault($pageInfo['caption:'.$currentLanguage], $item['p_caption']);
        //$caption = getDefault($item['p_caption'], i18n('mci.unnamed'));
        $nodes[] = '<a href="'.getNodeUrl($item).'">'.$caption.'</a>';
      }
    return(implode($separator, $nodes));
  }

  class PageRenderer
  {
    var $pageTemplate;
    var $content;
    var $currentBlock;
    var $currentLanguage;
    var $nodePath;
    var $themePath;
    var $themeCfg;
    var $templateCfg;

    function PageRenderer(&$contentNode)
    {
      profiler_log('new PageRenderer('.$contentNode->nodeDS['p_key'].')');
      global $config;
      $_REQUEST['node'] = getDefault($_REQUEST['node'], $config['site']['home_node']); 
      $this->content = &$contentNode;
      $this->currentLanguage = $contentNode->currentLanguage;

      $this->pageTemplate =
        getDefault(str_replace('.manifest.php', '.php', $contentNode->nodeDS['p_template']),
          $config['site']['defaultpagetemplate']);

      // if this page is a list entry, we need to determine the page template
      // by looking at the list itself
      if ($this->content->nodeDS['p_ref_list'] > -1)
      {
        $listId = $this->content->nodeDS['p_ref_list'];
        $listDS = DB_GetDataset('settings', $listId);
        $listSettings = unserialize($listDS['s_content']);
        $this->pageTemplate =
          getDefault(str_replace('.manifest.php', '.php', $listSettings['detailtemplate']),
            $config['site']['defaultpagetemplate']);
      }
    }

    function displayPage()
    {
      global $config, $basePath, $site;
      
      $this->themePath = 'themes/'.$config['site']['theme'].'/';
      $themeConfigFile = $this->themePath.'theme.cfg.php';
      if (file_exists($themeConfigFile)) include($themeConfigFile);
      
      include('themes/'.$config['site']['theme'].'/pages/'.$this->pageTemplate);
      profiler_log('PageRenderer: page template done');
    }

    function getNodePath($tonode, $fromNode = null)
    {
      $thisNode = array();
      do
      {
        $thisNode = DB_GetDataset('pages', $tonode);
        if (sizeof($thisNode)>0)
          $nodeArray[] = $thisNode;
        $tonode = $thisNode['p_ref_parent'];
      } while (sizeof($thisNode)>0 && $tonode!= $fromNode);
      if (!is_array($nodeArray)) $nodeArray = array();
      return(array_reverse($nodeArray));
    }

    function getBreadCrumbs($node = 1,
      $separator = ' / ',
      $itemCallBack = 'stdBreadCrumbItems')
    {
      global $nodeArray;
      if (!isset($nodeArray))
        $nodeArray = $this->getNodePath($node);
      $this->nodePath = $nodeArray;
      return($itemCallBack($nodeArray, $separator));
    }

    function getField($fieldname, $inBlock = null)
    {
      if (!isset($inBlock))
      {
        $inBlock = $this->currentBlock;
      }
      $content = getDefault(
        $inBlock[$fieldname.':'.$this->currentLanguage],
        $inBlock[$fieldname]);
      return(stripslashes($content));
    }

    function getTextBody($fieldname, $inBlock = null)
    {
      $lines = explode("\n", $this->getField($fieldname, $inBlock));
      foreach ($lines as $line)
      {
        $line = trim($line);
        if (substr($line, 0, 1) == ':' && substr($line, -1, 1) == ':')
          $line = '<h3>'.substr($line, 1, -1).'</h3>';
        else if (substr($line, 0, 1) == '*')
          $line = '<ul><li>'.substr($line, 1).'</li></ul>';
        else
          $line .= '<br/>';
        $result .= $line;
      }
      return($result);
    }

    function renderBlocks($section = 'main', $contentNode = null)
    {
      profiler_log('PageRenderer->renderBlocks('.$section.') init');
      global $config;                                              
      if (isset($contentNode))
      {
        $tempcontent = $this->content;
        $this->content = $contentNode;
      }
      $blocklist = $this->content->getBlockList($section);
      $blockIndex = 0;
      if (is_array($blocklist))
        foreach ($blocklist as $block)
        {
          if ($block['title:'.$this->currentLanguage] != '' || $block['caption:'.$this->currentLanguage] != '')
            $blockIndex++;
          $this->currentBlock = $block;
          $blockName = getDefault($block['style'], $block['type'].'.php');
          $styleFileName = 'themes/'.$config['site']['theme'].'/blocks/'.$blockName;
          $defaultFileName = 'themes/default/blocks/'.$blockName;
          if ($block['type'] == 'list') include_once('lib/listblocks.php');
          if (file_exists($styleFileName)) include($styleFileName);
          else
          {
            $svThemePath = $this->themePath;
            $this->themePath = 'themes/default/';
            include($defaultFileName);
            $this->themePath = $svThemePath;
          }
          profiler_log('PageRenderer->renderBlocks('.$section.') '.$plainFileName.'/'.$styleFileName);
        }
      if (isset($contentNode))
      {
        $this->content = $tempcontent;
      }
      profiler_log('PageRenderer->renderBlocks('.$section.') done');
    }

    function _getMenuLevel($parentId, &$nodeArray, $level = 0,
      $indent = '', &$properties)
    {
      global $nodePath, $currentSiteID, $config;
      $query = 'SELECT * FROM '.getTableName('pages').' WHERE 
          p_ref_parent="'.$parentId.'" AND p_status != "H" AND 
          refSiteID = "'.$currentSiteID.'" AND p_ref_list = -1
        ORDER BY p_order,p_caption';
      $menuList = DB_GetList($query);

      $nextBranch = $nodeArray[$level]['p_key'];
      $lvlTemplate = getDefault($properties['template.level'.$level], $properties['template']);

      foreach ($menuList as $item)
      {
        $active = ''; $branch = '';
        if ($_REQUEST['node']==$item['p_key'])
          $active = '_active';

        $url = getNodeUrl($item);
        $pageInfo = unserialize($item['p_pageinfo']);
        $caption = getDefault($pageInfo['caption:'.$this->currentLanguage], $item['p_caption']);
        $caption = getDefault($caption, i18n('mci.unnamed'));

        if ($item['p_key'] == $nextBranch && $nextBranch!=1)
          $branch = '_branch';

        eval('?'.'>'.$lvlTemplate);

        if ($item['p_key'] == $nextBranch || $properties['openall']==true &&
          (($properties['depth']==0) || ($properties['depth'] < $level+1)))
        {
          $this->_getMenuLevel($nextBranch, $nodeArray, $level+1,
            $indent.'&nbsp;&nbsp;&nbsp;',
            $properties);
        }
      }
    }

    function renderMenu($properties)
    {
      profiler_log('PageRenderer->renderMenu() init');
      global $nodeArray, $config;
      if (!isset($nodeArray))
        $nodeArray = $this->getNodePath($_REQUEST['node'], $properties['parentlevel']);
      $properties['template'] = getDefault($properties['template'],
        '<div class="mainmenuitem mainmenuitem<'.'?= $active ?'.'>" 
         onclick="document.location=\'<'.'?= htmlentities($url) ?'.'>\';"
          ><a href="<'.'?= $url ?'.'>"><'.'?= $caption ?'.'></a></div>');
      $parentnode = getDefault($properties['parentlevel'], 0);
      $this->_getMenuLevel($parentnode, $nodeArray, 0, '',
        $properties);
      profiler_log('PageRenderer->renderMenu() done');
    }
  }

?>
